/*
 *      Copyright (C) 2009, David Delassus <linkdd62@gmail.com>
 *
 *      This program is free software; you can redistribute it and/or modify
 *      it under the terms of the GNU General Public License as published by
 *      the Free Software Foundation; either version 3 of the License, or
 *      (at your option) any later version.
 *
 *      This program is distributed in the hope that it will be useful,
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of
 *      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *      GNU General Public License for more details.
 *
 *      You should have received a copy of the GNU General Public License
 *      along with this program; if not, write to the Free Software
 *      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 *      MA 02110-1301, USA.
 */

#include "local.h"

/* When the page title change */
void webview_title_changed_cb ( WebKitWebView *webview,
                                WebKitWebFrame *frame,
                                gchar *title,
                                gpointer user_data)
{
    CreamView *obj = (CreamView *) user_data;
    gint i;

    /* search current page */
    for (i = 0; i < gtk_notebook_get_n_pages (GTK_NOTEBOOK (global.notebook)); ++i)
    {
        GtkWidget *tmp = gtk_notebook_get_nth_page (GTK_NOTEBOOK (global.notebook), i);

        if (tmp == obj->box)
        {
            /* change current tab title */
            gtk_notebook_set_tab_label (GTK_NOTEBOOK (global.notebook), tmp,
                notebook_create_tab (obj));

            /* change window name */
            if (title != NULL)
            {
                gtk_window_set_title (GTK_WINDOW (global.win),
                    g_strconcat (title, " - Cream ", VERSION, NULL)
                );
            }
            else
            {
                gtk_window_set_title (GTK_WINDOW (global.win), "Cream " VERSION);
            }

            break;
        }
    }
}

/* When the loading's percent change */
void webview_progress_changed_cb (  WebKitWebView *webview,
                                    gint progress,
                                    gpointer user_data)
{
    CreamView *obj = (CreamView *) user_data;

    const gchar *url = webkit_web_view_get_uri (webview);
    gchar *text = g_strconcat (_("Transfering data from "), url, NULL);
    cream_view_update_url (obj, text, FALSE);
    g_free (text);

#if GTK_CHECK_VERSION (2, 16, 0)
    gtk_entry_set_progress_fraction (GTK_ENTRY (obj->command_line), progress == 100 ? 0 : (double) progress / 100);
#endif

    cream_view_update_state (obj, progress);
}

/* When the user send a new URL to load */
void webview_load_committed_cb (WebKitWebView *webview,
                                WebKitWebFrame *frame,
                                gpointer user_data)
{
    CreamView *obj = (CreamView *) user_data;

    const gchar *url = webkit_web_view_get_uri (webview);
    gchar *text = g_strconcat (_("Waiting reply from "), url, NULL);
    cream_view_update_url (obj, text, FALSE);
    g_free (text);
}

/* When the URL loading is finish */
void webview_load_finished_cb ( WebKitWebView *webview,
                                WebKitWebFrame *frame,
                                gpointer user_data)
{
    CreamView *obj = (CreamView *) user_data;

    const gchar *url = webkit_web_view_get_uri (webview);
    cream_view_update_url (obj, url, FALSE);

    cream_view_script (obj, JS_SETUP);
    history_add_item (url, webkit_web_view_get_title (webview));
}

WebKitNavigationResponse webview_navigation_cb (
                                        WebKitWebView *web_view,
                                        WebKitWebFrame *frame,
                                        WebKitNetworkRequest *request,
                                        gpointer data)
{
    const gchar *uri = webkit_network_request_get_uri (request);
    CreamView *obj = (CreamView *) data;

    if (g_str_has_prefix (uri, "ftp://"))
    {
        if (1) /* isdir() */
        {
            cream_view_load_ftp (obj, uri);
            return WEBKIT_NAVIGATION_RESPONSE_IGNORE;
        }
        else
        {
            return WEBKIT_NAVIGATION_RESPONSE_DOWNLOAD;
        }
    }
    else
    {
        return WEBKIT_NAVIGATION_RESPONSE_ACCEPT;
    }
}

/* When the WebKit widget request a new window */
WebKitWebView *webview_new_window_cb (  WebKitWebView *webview,
                                        WebKitWebFrame *frame,
                                        gpointer user_data)
{
    CreamView *cream = cream_view_new ();

    /* open it in a new tab */
    gtk_notebook_append_page (GTK_NOTEBOOK (global.notebook), cream->box,
        notebook_create_tab (cream)
    );

    gtk_notebook_set_tab_reorderable (GTK_NOTEBOOK (global.notebook), cream->box, TRUE);

    /* return the new WebKit widget */
    return WEBKIT_WEB_VIEW (cream->webview);
}

gboolean webview_mimetype_cb (  WebKitWebView *webview,
                                WebKitWebFrame *frame,
                                WebKitNetworkRequest *request,
                                gchar *mime_type,
                                WebKitWebPolicyDecision *decision,
                                gpointer user_data)
{
    if (webkit_web_view_can_show_mime_type (webview, mime_type))
        webkit_web_policy_decision_use (decision);
    else
        webkit_web_policy_decision_download (decision);

    return TRUE;
}

/* When a download is requested */
gboolean webview_download_cb (  WebKitWebView *webview,
                                WebKitDownload *download,
                                gpointer user_data)
{
    return FALSE;
}

/* When the user ove the mouse hover a link */
void webview_hoverlink_cb ( WebKitWebView *webview,
                            gchar *title,
                            gchar *link,
                            gpointer data)
{
    CreamView *obj = (CreamView *) data;

    if (link == NULL)
        cream_view_update_url (obj, webkit_web_view_get_uri (webview), FALSE);
    else
        cream_view_update_url (obj, link, TRUE);
}

/* console message (JavaScript) */
gboolean webview_console_cb (   WebKitWebView *webview,
                                gchar *message,
                                gint line,
                                gchar *source,
                                gpointer user_data)
{
    CreamView *obj = (CreamView *) user_data;

    if (!strcmp (message, "hintmode_off") || !strcmp (message, "insertmode_off"))
    {
        webkit_web_view_unmark_text_matches (WEBKIT_WEB_VIEW (obj->webview));
        echo (obj, "");
    }
    else if (!strcmp (message, "insertmode_on"))
    {
        echo (obj, "-- INSERT --");
    }

    return TRUE;
}

/* add items to the WebKit menu */
void webview_populate_menu_cb ( WebKitWebView *web_view,
                                GtkMenu *menu,
                                gpointer data)
{
    GtkWidget *item;

    /* separator */
    item = gtk_separator_menu_item_new ();
    gtk_menu_shell_append (GTK_MENU_SHELL (menu), item);

    /* add to bookmarks */
    item = gtk_menu_item_new_with_label (_("Add to bookmarks"));
    g_signal_connect (item, "activate", (GCallback) bookmark_cb, data);
    gtk_menu_shell_append (GTK_MENU_SHELL (menu), item);

    /* show page informations */
    item = gtk_image_menu_item_new_with_label (_("Show page informations"));
    gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM (item),
        gtk_image_new_from_stock (GTK_STOCK_INFO, GTK_ICON_SIZE_MENU));

    gtk_menu_shell_append (GTK_MENU_SHELL (menu), item);

    /* show page properties */
    item = gtk_image_menu_item_new_with_label (_("Show page properties"));
    gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM (item),
        gtk_image_new_from_stock (GTK_STOCK_PROPERTIES, GTK_ICON_SIZE_MENU));

    gtk_menu_shell_append (GTK_MENU_SHELL (menu), item);

    /* print page */
    item = gtk_image_menu_item_new_with_label (_("Print page"));
    gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM (item),
        gtk_image_new_from_stock (GTK_STOCK_PRINT, GTK_ICON_SIZE_MENU));

    g_signal_connect (item, "activate", (GCallback) print_page_cb, data);
    gtk_menu_shell_append (GTK_MENU_SHELL (menu), item);

    gtk_widget_show_all (GTK_WIDGET (menu));
}


void webview_scroll_cb (GtkAdjustment *adjustment,
                        gpointer user_data)
{
    CreamView *obj = (CreamView *) user_data;
    cream_view_update_state (obj, 100);
}

/* When the user close a tab */
void notebook_close_page_cb (GtkWidget *widget, gpointer data)
{
    CreamView *obj = (CreamView *) data;

    gtk_notebook_remove_page (GTK_NOTEBOOK (global.notebook),
        gtk_notebook_page_num (GTK_NOTEBOOK (global.notebook), obj->box));

    if (0 == gtk_notebook_get_n_pages (GTK_NOTEBOOK (global.notebook)))
        add_cream_view_to_notebook (global.homepage);
}

/* Create the WebInspector window */
WebKitWebView *create_inspect_win_cb (  WebKitWebInspector *web_inspector,
                                        WebKitWebView *web_view,
                                        gpointer data)
{
    GtkWidget *inspect_win;
    GtkWidget *inspect_page;

    inspect_win = gtk_window_new (GTK_WINDOW_TOPLEVEL);
    gtk_window_set_default_size (GTK_WINDOW (inspect_win), 800, 600);

    gtk_window_set_title (GTK_WINDOW (inspect_win), g_strconcat (
        "WebInspector - ",  webkit_web_view_get_uri (web_view), "- Cream ", VERSION, NULL)
    );

    inspect_page = webkit_web_view_new ();
    g_signal_connect (inspect_page, "console-message", (GCallback) webview_console_cb, data);
    gtk_container_add (GTK_CONTAINER (inspect_win), inspect_page);

    gtk_widget_show_all (inspect_win);

    return WEBKIT_WEB_VIEW (inspect_page);
}

gboolean inputbox_key_press_cb (GtkWidget *widget,
                                GdkEventKey *event,
                                gpointer data)
{
    CreamView *obj = (CreamView *) data;
    extern GSList *cmd_history;
    extern int current_item;
    GString *tmp;

    if (event->keyval == GDK_Up)
    {
        if (current_item > 0)
        {
            current_item--;
            tmp = (GString *) g_slist_nth_data (cmd_history, current_item);
            echo (obj, "%s", tmp->str);
        }

        gtk_widget_grab_focus (obj->command_line);
        gtk_editable_set_position (GTK_EDITABLE (obj->command_line), -1);

        return TRUE;
    }
    else if (event->keyval == GDK_Down)
    {
        if (current_item < (g_slist_length (cmd_history) - 1))
        {
            current_item++;
            tmp = (GString *) g_slist_nth_data (cmd_history, current_item);
            echo (obj, "%s", tmp->str);
        }
        else if (current_item == (g_slist_length (cmd_history) - 1))
        {
            current_item++;
            echo (obj, "");
        }

        gtk_widget_grab_focus (obj->command_line);
        gtk_editable_set_position (GTK_EDITABLE (obj->command_line), -1);

        return TRUE;
    }
    else if (event->keyval == GDK_Escape)
    {
        WebKitWebSettings *settings = webkit_web_view_get_settings (WEBKIT_WEB_VIEW (obj->webview));
        g_object_set (settings, "enable-caret-browsing", FALSE, NULL);
        webkit_web_view_set_settings (WEBKIT_WEB_VIEW (obj->webview), settings);

        cream_view_complete (obj, HideCompletion);
        echo (obj, "");
        return TRUE;
    }
    else if (event->keyval == GDK_Tab)
    {
        cream_view_complete (obj, DirectionNext);
        return TRUE;
    }
    else if (event->keyval == GDK_ISO_Left_Tab)
    {
        cream_view_complete (obj, DirectionPrev);
        return TRUE;
    }

    return FALSE;
}

/* Keybindings */
gboolean window_key_press ( GtkWidget *widget,
                            GdkEventKey *event,
                            gpointer data)
{
    CreamView *view = notebook_get_creamview (-1);
    extern Keybinding shortkeys[];
    gint i;

    /* FIXME: Why 16 ? It's the only solution I found, and it works for me.
     * Please report any bugs of keybindings.
     */

    guint state = event->state - 16;

    if (state == 0)
    {
        if (strchr (global.modkeys, event->keyval)
            && global.current_modkey != event->keyval)
        {
            global.current_modkey = event->keyval;
            return TRUE;
        }
    }

    for (i = 0; shortkeys[i].id != NULL; ++i)
    {
        if (shortkeys[i].modmask & state
            && (shortkeys[i].modkey == global.current_modkey
                || (!shortkeys[i].modkey && !global.current_modkey)
                || shortkeys[i].modkey == GDK_VoidSymbol)
            && shortkeys[i].key == event->keyval
            && shortkeys[i].func)
        {
            shortkeys[i].func (view, shortkeys[i].argument);
            global.current_modkey = 0;
            return TRUE;
        }
    }

    return FALSE;
}

void notebook_page_added_cb (   GtkNotebook *notebook,
                                GtkWidget *child,
                                guint page_num,
                                gpointer user_data)
{
    gint i;
    for (i = 0; i < gtk_notebook_get_n_pages (notebook); ++i)
    {
        CreamView *view = notebook_get_creamview (i);
        cream_view_update_state (view, 100);
    }

    gtk_widget_show_all (GTK_WIDGET (notebook));
    gtk_notebook_set_current_page (notebook, page_num);
}

void notebook_page_removed_cb ( GtkNotebook *notebook,
                                GtkWidget *child,
                                guint page_num,
                                gpointer user_data)
{
    gint i;

    for (i = 0; i < gtk_notebook_get_n_pages (notebook); ++i)
    {
        CreamView *view = notebook_get_creamview (i);
        cream_view_update_state (view, 100);
    }

    if (gtk_notebook_get_n_pages (notebook) == 0)
        add_cream_view_to_notebook (global.homepage);
}

void notebook_page_reordered_cb (   GtkNotebook *notebook,
                                    GtkWidget *child,
                                    guint page_num,
                                    gpointer user_data)
{
    gint i;
    for (i = 0; i < gtk_notebook_get_n_pages (notebook); ++i)
    {
        CreamView *view = notebook_get_creamview (i);
        cream_view_update_state (view, 100);
    }
}

void notebook_switch_page_cb (  GtkNotebook *notebook,
                                GtkWidget *child,
                                guint page_num,
                                gpointer user_data)
{
    CreamView *view;
    const gchar *title;
    gint i;

    for (i = 0; i < gtk_notebook_get_n_pages (notebook); ++i)
    {
        view = notebook_get_creamview (i);
        cream_view_update_state (view, 100);
    }

    view = notebook_get_creamview (page_num);
    title = webkit_web_view_get_title (WEBKIT_WEB_VIEW (view->webview));

    if (title != NULL)
    {
        gtk_window_set_title (GTK_WINDOW (global.win),
            g_strconcat (title, " - Cream ", VERSION, NULL)
        );
    }
    else
    {
        gtk_window_set_title (GTK_WINDOW (global.win), "Cream " VERSION);
    }
}

void print_page_cb (GtkWidget *widget, gpointer data)
{
    CreamView *view = (CreamView *) data;

    webkit_web_frame_print (webkit_web_view_get_main_frame (
        WEBKIT_WEB_VIEW (view->webview))
    );
}

void bookmark_cb (GtkWidget *widget, gpointer data)
{
    CreamView *view = (CreamView *) data;
    const gchar *title = webkit_web_view_get_title (WEBKIT_WEB_VIEW (view->webview));
    const gchar *uri = webkit_web_view_get_uri (WEBKIT_WEB_VIEW (view->webview));

    bookmarks_add_item (uri, title);
}
