#include "local.h"

typedef struct
{
    GHashTable *memory;
    gchar *cache_path;
} Favicon;

static void g_object_maybe_unref (gpointer obj)
{
    if (obj)
        g_object_unref (obj);
    return;
}

static gchar *favicon_get_cached_path ( Favicon *obj,
                                        const gchar *uri,
                                        const gchar *subfolder)
{
    gchar* cache_path;
    gchar* checksum;
    gchar* extension;
    gchar* cached_filename;
    gchar* cached_path;

    if (subfolder)
        cache_path = g_build_filename (obj->cache_path, subfolder, NULL);
    else
        cache_path = obj->cache_path;

    g_mkdir_with_parents (cache_path, 0700);
    checksum = g_compute_checksum_for_string (G_CHECKSUM_MD5, uri, -1);

    extension = g_strrstr (uri, ".");
    cached_filename = g_strdup_printf ("%s%s", checksum, extension ? extension : "");

    g_free (checksum);

    cached_path = g_build_filename (cache_path, cached_filename, NULL);
    g_free (cached_filename);

    if (subfolder)
        g_free (cache_path);

    return cached_path;
}

GtkWidget *favicon_load_from_uri (const gchar *uri)
{
    Favicon temp;
    guint i;
    gchar *icon_file;
    gchar *icon_uri;
    GdkPixbuf *icon = NULL;

    temp.memory = g_hash_table_new_full (g_str_hash, g_str_equal,
                        g_free, g_object_maybe_unref);
    temp.cache_path = g_build_filename (g_get_user_cache_dir (), "cream", NULL);

    if (uri && (g_str_has_prefix (uri, "http://")
        || g_str_has_prefix (uri, "https://")
        || g_str_has_prefix (uri, "file://")))
    {
        i = (g_str_has_prefix (uri, "https://") ? 9 : 8);

        for (; uri[i] != 0 && uri[i] != '/'; ++i);

        if (uri[i] == '/')
        {
            icon_uri = g_strdup (uri);
            icon_uri[i] = 0;
            icon_uri = g_strdup_printf ("%s/favicon.ico", icon_uri);
        }
        else
            icon_uri = g_strdup_printf ("%s/favicon.ico", uri);

        icon_file = favicon_get_cached_path (&temp, icon_uri, "icons");

        if (g_hash_table_lookup_extended (temp.memory, icon_file,
                NULL, (gpointer) &icon))
        {
            g_free (icon_file);

            if (icon)
                g_object_ref (icon);
        }
        else if ((icon = gdk_pixbuf_new_from_file (icon_file, NULL)))
            g_free (icon_file);

        else
        {
            gchar *arg_v[] =
            {
                "/usr/bin/wget",
                icon_uri,
                "-O",
                icon_file,
                NULL
            };

            g_spawn_async (NULL, arg_v, NULL,
                G_SPAWN_STDOUT_TO_DEV_NULL | G_SPAWN_STDERR_TO_DEV_NULL,
                NULL, NULL, NULL, NULL);

        }
        g_free (icon_uri);
    }

    if (!icon)
    {
        return gtk_image_new_from_stock (GTK_STOCK_FILE, GTK_ICON_SIZE_MENU);
    }
    else
    {
        gint w = 16;
        gint h = 16;

        gtk_icon_size_lookup (GTK_ICON_SIZE_MENU, &w, &h);
        icon = gdk_pixbuf_scale_simple (icon, w, h, GDK_INTERP_BILINEAR);

        return gtk_image_new_from_pixbuf (icon);
    }

    return NULL;
}
