/*
 *      Copyright (C) 2009, David Delassus <linkdd62@gmail.com>
 *
 *      This program is free software; you can redistribute it and/or modify
 *      it under the terms of the GNU General Public License as published by
 *      the Free Software Foundation; either version 3 of the License, or
 *      (at your option) any later version.
 *
 *      This program is distributed in the hope that it will be useful,
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of
 *      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *      GNU General Public License for more details.
 *
 *      You should have received a copy of the GNU General Public License
 *      along with this program; if not, write to the Free Software
 *      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 *      MA 02110-1301, USA.
 */

#include "local.h"

ini_t *ini_open (char *file)
{
    ini_t *ini = calloc (sizeof (ini_t), 1);

    ini->filename = g_strdup (file);
    ini->fp = fopen (file, "r");
    if (ini->fp == NULL)
    {
        free (ini);
        return NULL;
    }

    return ini;
}

int ini_read (ini_t *file)
{
    char key[MAX_STRING], s[MAX_STRING], *t;
    int i;

    while (!feof (file->fp))
    {
        *s = 0;

        (void) fscanf (file->fp, "%127[^\n#]s", s);
        (void) fscanf (file->fp, "%*[^\n]s");
        fgetc (file->fp);

        file->line++;

        if (strchr (s, '='))
        {
            sscanf (s, "%[^ =]s", key);

            if ((t = strchr (key, '.')))
            {
                *t = 0;
                strcpy (file->section, key);
                t++;
            }
            else
            {
                strcpy (file->section, file->c_section);
                t = key;
            }

            sscanf (t, "%s", file->key);
            t = strchr (s, '=') + 1;

            for (i = 0; t[i] == ' '; ++i);

            strcpy (file->value, &t[i]);

            for (i = strlen (file->value) - 1; file->value[i] == 32; --i)
                file->value[i] = 0;

            return 1;
        }
        else if ((t = strchr (s, '[')))
        {
            strcpy (file->c_section, t + 1);
            t = strchr (file->c_section, ']');
            *t = 0;
        }
    }

    return 0;
}

void ini_close (ini_t *file)
{
    fclose (file->fp);
    free (file);
    return;
}

int config_load (char *filename)
{
    ini_t *ini;

    ini = ini_open (filename);
    if (ini == NULL) return -1;

    while (ini_read (ini))
    {
        if (!strcasecmp (ini->section, "global"))
        {
            if (!strcasecmp (ini->key, "homepage"))
            {
                if (strcasecmp (ini->value, "default"))
                {
                    global.homepage = g_strdup (ini->value);
                }
            }
            else if (!strcasecmp (ini->key, "encoding"))
            {
                if (strcasecmp (ini->value, "default"))
                {
                    global.encoding = g_strdup (ini->value);
                }
            }
            else if (!strcasecmp (ini->key, "javascript"))
            {
                if (!strcasecmp (ini->value, "yes"))
                {
                    global.javascript = TRUE;
                }
                else if (!strcasecmp (ini->value, "no"))
                {
                    global.javascript = FALSE;
                }
            }
            else
            {
                fprintf (stderr, "Error: \"%s\":%d -> Unknow key '%s'\n",
                    filename, ini->line, ini->key);
            }
        }
        else if (!strcasecmp (ini->section, "commands"))
        {
            extern Command cmdlist[];
            gint i;

            for (i = 0; cmdlist[i].id != NULL; ++i)
            {
                if (!strcasecmp (ini->key, cmdlist[i].id))
                {
                    cmdlist[i].cmd = g_strdup (ini->value);
                    break;
                }
            }
        }
        else if (!strcasecmp (ini->section, "keybindings"))
        {
            extern Keybinding shortkeys[];
            gint i;

            for (i = 0; shortkeys[i].id != NULL; ++i)
            {
                if (!strcasecmp (ini->key, shortkeys[i].id))
                {
                    path_to_key (ini->value,
                        &shortkeys[i].modmask,
                        &shortkeys[i].modkey,
                        &shortkeys[i].key
                    );

                    break;
                }
            }
        }
        else
        {
            fprintf (stderr, "Error: \"%s\":%d -> Unknow section '%s'\n",
                filename, ini->line, ini->section);
        }
    }

    ini_close (ini);

    return 0;
}
