/*
 *      Copyright (C) 2009, David Delassus <linkdd62@gmail.com>
 *
 *      This program is free software; you can redistribute it and/or modify
 *      it under the terms of the GNU General Public License as published by
 *      the Free Software Foundation; either version 3 of the License, or
 *      (at your option) any later version.
 *
 *      This program is distributed in the hope that it will be useful,
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of
 *      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *      GNU General Public License for more details.
 *
 *      You should have received a copy of the GNU General Public License
 *      along with this program; if not, write to the Free Software
 *      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 *      MA 02110-1301, USA.
 */

#include "local.h"

Keybinding shortkeys[] =
{
    /* identifier       modmask             modkey  key             function        argument */
    { "insert",         0,                  0,      GDK_Insert,     key_switch,     InputMode },
    { "echap",          0,                  0,      GDK_Escape,     key_switch,     ViewMode },
    { "hints",          GDK_CONTROL_MASK,   0,      GDK_f,          key_switch,     HintsMode },
    { "caret",          GDK_CONTROL_MASK,   0,      GDK_Insert,     key_switch,     CaretMode },
    { "previous",       0,                  0,      GDK_BackSpace,  key_previous,   NoArguments },
    { "open",           0,                  0,      GDK_o,          key_open,       NoArguments },
    { "Open",           GDK_SHIFT_MASK,     0,      GDK_O,          key_open,       InsertCurrentURL },
    { "tabopen",        0,                  0,      GDK_t,          key_tabopen,    NoArguments },
    { "Tabopen",        GDK_SHIFT_MASK,     0,      GDK_T,          key_tabopen,    InsertCurrentURL, },
    { "new-tab",        GDK_CONTROL_MASK,   0,      GDK_t,          key_tabopen,    Tabopen },
    { "close",          GDK_CONTROL_MASK,   0,      GDK_w,          key_close,      NoArguments },
    { "refresh",        0,                  0,      GDK_F5,         key_refresh,    NoArguments },
    { "bypass-cache",   0,                  0,      GDK_F6,         key_refresh,    RefreshByPassCache },
    { "zoom-in",        0,                  0,      GDK_plus,       key_zoom,       ZoomIn },
    { "zoom-out",       0,                  0,      GDK_minus,      key_zoom,       ZoomOut },
    { "yank",           0,                  0,      GDK_y,          key_yank,       NoArguments },
    { "paste",          0,                  0,      GDK_p,          key_paste,      NoArguments },
    { "Paste",          GDK_SHIFT_MASK,     0,      GDK_P,          key_paste,      Tabopen },
    { "quit",           GDK_CONTROL_MASK,   0,      GDK_q,          key_quit,       NoArguments },
    { NULL, 0, 0, 0, NULL, 0 }
};

void path_to_key (const gchar *path, guint *modmask, guint *modkey, guint *key)
{
    gchar **id = g_strsplit (path, "+", 0);
    gint i;

    for (i = 0; i < g_strv_length (id); ++i)
    {
        /* modmask */
        if (!strcasecmp (id[i], "<ctrl>"))
        {
            *modmask |= GDK_CONTROL_MASK;
        }
        else if (!strcasecmp (id[i], "<shift>"))
        {
            *modmask |= GDK_SHIFT_MASK;
        }
        else if (!strcasecmp (id[i], "<alt>"))
        {
            *modmask |= GDK_MOD1_MASK;
        }
        else if (!strcasecmp (id[i], "<super>"))
        {
            *modmask |= GDK_SUPER_MASK;
        }
        /* key */
        else
        {
            *key |= gdk_keyval_from_name (id[i]);
        }
    }

    g_strfreev (id);
}

void key_switch (CreamView *obj, Arg arg)
{
    WebKitWebSettings *settings = webkit_web_view_get_settings (WEBKIT_WEB_VIEW (obj->webview));

    if (arg == InputMode)
    {
        echo (obj, ":");
        gtk_widget_grab_focus (obj->command_line);
        gtk_editable_set_position (GTK_EDITABLE (obj->command_line), -1);
    }
    else if (arg == ViewMode)
    {
        g_object_set (settings, "enable-caret-browsing", FALSE, NULL);
        gtk_widget_grab_focus (obj->webview);
        echo (obj, "");
    }
    else if (arg == HintsMode)
    {
        cream_view_script (obj, "show_hints ()");
    }
    else if (arg == CaretMode)
    {
        g_object_set (settings, "enable-caret-browsing", TRUE, NULL);
        echo (obj, "-- CARET --");
    }

    webkit_web_view_set_settings (WEBKIT_WEB_VIEW (obj->webview), settings);
}

void key_previous (CreamView *obj, Arg arg)
{
    webkit_web_view_go_back (WEBKIT_WEB_VIEW (obj->webview));
}

void key_open (CreamView *obj, Arg arg)
{
    gchar *str;

    if (arg == InsertCurrentURL)
    {
        str = g_strconcat (get_command ("open"),
            webkit_web_view_get_uri (WEBKIT_WEB_VIEW (obj->webview)),
            NULL
        );
    }
    else
    {
        str = get_command ("open");
    }

    echo (obj, ":%s", str);
    gtk_widget_grab_focus (obj->command_line);
    gtk_editable_set_position (GTK_EDITABLE (obj->command_line), -1);

    g_free (str);
}

void key_tabopen (CreamView *obj, Arg arg)
{
    gchar *str;

    if (arg == InsertCurrentURL)
    {
        str = g_strconcat (get_command ("tabopen"),
            webkit_web_view_get_uri (WEBKIT_WEB_VIEW (obj->webview)),
            NULL
        );
    }
    else if (arg == Tabopen)
    {
        add_cream_view_to_notebook (global.homepage);
        return;
    }
    else
    {
        str = get_command ("tabopen");
    }

    echo (obj, ":%s", str);
    gtk_widget_grab_focus (obj->command_line);
    gtk_editable_set_position (GTK_EDITABLE (obj->command_line), -1);

    g_free (str);
}

void key_close (CreamView *obj, Arg arg)
{
    global.trash = g_slist_append (global.trash, g_string_new (
        webkit_web_view_get_uri (WEBKIT_WEB_VIEW (obj->webview))
    ));

    gtk_notebook_remove_page (GTK_NOTEBOOK (global.notebook),
        gtk_notebook_current_page (GTK_NOTEBOOK (global.notebook))
    );
}

void key_refresh (CreamView *obj, Arg arg)
{
    if (arg == RefreshByPassCache)
    {
        webkit_web_view_reload_bypass_cache (WEBKIT_WEB_VIEW (obj->webview));
    }
    else
    {
        webkit_web_view_reload (WEBKIT_WEB_VIEW (obj->webview));
    }
}

void key_zoom (CreamView *obj, Arg arg)
{
    if (arg == ZoomIn)
    {
        webkit_web_view_zoom_in (WEBKIT_WEB_VIEW (obj->webview));
    }
    else if (arg == ZoomOut)
    {
        if ((gint) (webkit_web_view_get_zoom_level (WEBKIT_WEB_VIEW (obj->webview)) * 100.0) > 20)
        {
            webkit_web_view_zoom_out (WEBKIT_WEB_VIEW (obj->webview));
        }
    }
}

void key_yank (CreamView *obj, Arg arg)
{
    const gchar *uri = webkit_web_view_get_uri (WEBKIT_WEB_VIEW (obj->webview));
    echo (obj, "Yanked %s", uri);
    gtk_clipboard_set_text (global.clipboard, uri, -1);
}

void key_paste (CreamView *obj, Arg arg)
{
    gchar *uri = gtk_clipboard_wait_for_text (global.clipboard);

    if (arg == Tabopen)
        add_cream_view_to_notebook (uri);
    else
        webkit_web_view_open (WEBKIT_WEB_VIEW (obj->webview), uri);
}

void key_quit (CreamView *obj, Arg arg)
{
    quit (NULL, NULL);
}
