/*
 *      Copyright (C) 2009, David Delassus <linkdd62@gmail.com>
 *
 *      This program is free software; you can redistribute it and/or modify
 *      it under the terms of the GNU General Public License as published by
 *      the Free Software Foundation; either version 3 of the License, or
 *      (at your option) any later version.
 *
 *      This program is distributed in the hope that it will be useful,
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of
 *      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *      GNU General Public License for more details.
 *
 *      You should have received a copy of the GNU General Public License
 *      along with this program; if not, write to the Free Software
 *      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 *      MA 02110-1301, USA.
 */

#ifndef __LOCAL_H
#define __LOCAL_H

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <signal.h>
#include <time.h>

#include <gtk/gtk.h>
#include <gdk/gdkkeysyms.h>
#include <webkit/webkit.h>
#include <JavaScriptCore/JavaScript.h>

#if !WEBKIT_CHECK_VERSION (1, 1, 6)
#   error You must have Webkit 1.1.6 or either
#endif

#if !GTK_CHECK_VERSION (2, 14, 0)
#   error You must have GTK+ 2.14.0 or either
#endif

#include "CreamView.h"
#include "callbacks.h"
#include "command.h"
#include "keybindings.h"
#include "ini.h"
#include "history.h"
#include "bookmarks.h"
#include "ftplib.h"
#include "html.h"

#include <locale.h>
#include <config.h>
#include "gettext.h"
#define _(string)   g_locale_to_utf8 (gettext (string), -1, NULL, NULL, NULL)

#define RESSOURCE               PREFIX "/share/cream/"
#define HISTORY_FILE            g_strdup_printf ("%s/.cream/history", g_get_home_dir ())
#define BOOKMARKS_FILE          g_strdup_printf ("%s/.cream/bookmarks", g_get_home_dir ())
#define SESSION_FILE            g_strdup_printf ("%s/.cream/session", g_get_home_dir ())
#define COOKIE_FILE             g_strdup_printf ("%s/.cream/cookie.txt", g_get_home_dir ())
#define CONFIG_FILE             g_strdup_printf ("%s/.cream/config", g_get_home_dir ())

typedef struct
{
    GtkWidget *win;
    GtkWidget *notebook;
    SoupCookieJar *cookies;
    GtkClipboard *clipboard;
    GSList *trash;

    gchar *homepage;
    gchar *encoding;
    gboolean javascript;

    char *modkeys;
    char current_modkey;

    int code;
} CreamGlobal;

extern CreamGlobal global;

GtkWidget *notebook_create_tab (CreamView *child);
GtkWidget *favicon_load_from_uri (const gchar *uri);
void add_cream_view_to_notebook (const gchar *url);
void quit (GtkWidget *emitter, gpointer data);

CreamView *notebook_get_creamview (gint i);
gchar *str_replace (const gchar *string, const gchar *substr, const gchar *replace);

#endif /* __LOCAL_H */
